@@ -269,6 +269,9 @@ impl RowGroupMetaData {
             total_byte_size: self.total_byte_size,
             num_rows: self.num_rows,
             sorting_columns: None,
+            file_offset: None,
+            total_compressed_size: None,
+            ordinal: None,
         }
     }
 }
@@ -517,6 +520,7 @@ impl ColumnChunkMetaData {
             dictionary_page_offset: self.dictionary_page_offset,
             statistics: statistics::to_thrift(self.statistics.as_ref()),
             encoding_stats: None,
+            bloom_filter_offset: None,
         };
 
         ColumnChunk {
@@ -527,6 +531,8 @@ impl ColumnChunkMetaData {
             offset_index_length: None,
             column_index_offset: None,
             column_index_length: None,
+            crypto_metadata: None,
+            encrypted_column_metadata: None,
         }
     }
 }
